 #!/usr/bin/perl -w
 
use strict;

BEGIN {

    if( index($^O, 'Win') >= 0 ) {   # MSWin32 (and not darwin, cygwin, ...)
        require Win32::SerialPort;
        import  Win32::SerialPort;
    } else {
        require Device::SerialPort;
        import  Device::SerialPort;
    }
}

$|++; # disable buffering

print <<WARR_END;

*** No Warranty
***
*** This program is distributed in the hope that it will be useful,
*** but is provided AS IS with ABSOLUTELY NO WARRANTY;
*** The entire risk as to the quality and performance
*** of the program is with you. Should the program prove defective,
*** you assume the cost of all necessary servicing, repair or correction.
*** In no event will any of the developers, or any other party,
*** be liable to anyone for damages arising out of the use or inability to use the program.

WARR_END

my ($serial, $count, $filehnd, $cur, $test, $line);
my ($dev, $filename) = @ARGV;

die "Error: not enough parameters! $0 <serial port> <flash file>" unless $dev and $filename;

my $ostype = $^O;
if (index($ostype, 'Win') >= 0) {
	$serial = Win32::SerialPort->new($dev)
} else {
	$serial = Device::SerialPort->new($dev)
}
die ("Error: can't open serial port. Typo? (e.g. COM1, /dev/ttyUSB0)") unless ref $serial;

$serial->baudrate(115200);
$serial->databits(8);
$serial->purge_all();
$serial->rts_active(0);
$serial->dtr_active(0);
$serial->read_const_time(1000);

open($filehnd, $filename) or die "Error: File not found.";
my @temp = <$filehnd>;
close($filehnd);

$count = @temp;

for $line (@temp) {

	$cur++;

	$line =~ s/\r\n//g; # get rid of any CR and LF
	print "$cur/$count: '", substr($line, 0, 39), length($line) > 40 ? '...' : '', "' ";
	if ($line =~ /^#.*/) {
		print "Comment not transferred.\n";
		next;
	}

	$serial->write("$line\r"); # append a CR 

	# waiting for response
	my ($try, $summe, $buffer) = (0, 0, '');
	while ($buffer !~ /\+/) {
		my ($anzahl, $saw) = $serial->read(length($line) + 2);
		$summe += $anzahl;	
		$buffer = "$buffer$saw";
		print '.';
		die "Timeout!" if $try++ > 10;
	}

	if ($buffer =~ /$line/) {
		print " OK\n"
	} else {
		print "\n  Last Response was: $buffer\n";
		die "Error: unknown transmission error.\n";
	}

	if ($cur == $count){ 
		print "READY!\n";
		last;
	}

	# power nap every 2000 lines
	sleep(1) unless $cur % 2000;
}

